@@ -1,4 +1,4 @@
-/*	$NetBSD: error.h,v 1.12 1998/07/28 11:41:53 mycroft Exp $	*/
+/*	$NetBSD: error.h,v 1.14 2001/02/04 19:52:06 christos Exp $	*/
 
 /*-
  * Copyright (c) 1991, 1993
@@ -82,7 +82,6 @@ extern int exception;
 
 extern volatile int suppressint;
 extern volatile int intpending;
-extern char *commandname;	/* name of command--printed on error */
 
 #define INTOFF suppressint++
 #define INTON { if (--suppressint == 0 && intpending) onint(); }
@@ -92,9 +91,9 @@ extern char *commandname;	/* name of command--printed on error */
 
 void exraise __P((int)) __attribute__((__noreturn__));
 void onint __P((void));
-void error __P((char *, ...)) __attribute__((__noreturn__));
-void exerror __P((int, char *, ...)) __attribute__((__noreturn__));
-char *errmsg __P((int, int));
+void error __P((const char *, ...)) __attribute__((__noreturn__));
+void exerror __P((int, const char *, ...)) __attribute__((__noreturn__));
+const char *errmsg __P((int, int));
 
 
 /*
