@@ -1,4 +1,4 @@
-/*	$NetBSD: jobs.c,v 1.30 1999/04/05 14:59:35 mycroft Exp $	*/
+/*	$NetBSD: jobs.c,v 1.36 2000/05/22 10:18:47 elric Exp $	*/
 
 /*-
  * Copyright (c) 1991, 1993
@@ -41,7 +41,7 @@
 #if 0
 static char sccsid[] = "@(#)jobs.c	8.5 (Berkeley) 5/4/95";
 #else
-__RCSID("$NetBSD: jobs.c,v 1.30 1999/04/05 14:59:35 mycroft Exp $");
+__RCSID("$NetBSD: jobs.c,v 1.36 2000/05/22 10:18:47 elric Exp $");
 #endif
 #endif /* not lint */
 
@@ -100,7 +100,7 @@ STATIC int dowait __P((int, struct job *));
 STATIC int onsigchild __P((void));
 STATIC int waitproc __P((int, int *));
 STATIC void cmdtxt __P((union node *));
-STATIC void cmdputs __P((char *));
+STATIC void cmdputs __P((const char *));
 
 
 /*
@@ -575,7 +575,7 @@ forkshell(jp, n, mode)
 	int pid;
 	int pgrp;
 	const char *devnull = _PATH_DEVNULL;
-	/* const */ char *nullerr = "Can't open %s";
+	const char *nullerr = "Can't open %s";
 
 	TRACE(("forkshell(%%%d, 0x%lx, %d) called\n", jp - jobtab, (long)n,
 	    mode));
@@ -730,19 +730,22 @@ waitforjob(jp)
 #endif
 	else
 		st = WTERMSIG(status) + 128;
+#if JOBS
+	if (jp->jobctl) {
+		/*
+		 * This is truly gross.
+		 * If we're doing job control, then we did a TIOCSPGRP which
+		 * caused us (the shell) to no longer be in the controlling
+		 * session -- so we wouldn't have seen any ^C/SIGINT.  So, we
+		 * intuit from the subprocess exit status whether a SIGINT
+		 * occured, and if so interrupt ourselves.  Yuck.  - mycroft
+		 */
+		if (WIFSIGNALED(status) && WTERMSIG(status) == SIGINT)
+			raise(SIGINT);
+	}
+#endif
 	if (! JOBS || jp->state == JOBDONE)
 		freejob(jp);
-#if 0
-	/*
-	 * XXXX
-	 * Why was this here?  It causes us to lose SIGINTs unless the current
-	 * command happens to also catch the SIGINT and exit with the right
-	 * status.  I don't see how that can possibly be correct.  -- mycroft
-	 */
-	CLEAR_PENDING_INT;
-	if (WIFSIGNALED(status) && WTERMSIG(status) == SIGINT)
-		raise(SIGINT);
-#endif
 	INTON;
 	return st;
 }
@@ -975,7 +978,7 @@ cmdtxt(n)
 	{
 	union node *np;
 	struct nodelist *lp;
-	char *p;
+	const char *p;
 	int i;
 	char s[2];
 
@@ -1100,9 +1103,10 @@ redir:
 
 STATIC void
 cmdputs(s)
-	char *s;
+	const char *s;
 	{
-	char *p, *q;
+	const char *p;
+	char *q;
 	char c;
 	int subtype = 0;
 
