@@ -1,4 +1,4 @@
-/*	$NetBSD: main.c,v 1.33 1999/03/27 13:46:19 christos Exp $	*/
+/*	$NetBSD: main.c,v 1.40 2001/02/04 19:52:06 christos Exp $	*/
 
 /*-
  * Copyright (c) 1991, 1993
@@ -46,10 +46,11 @@ __COPYRIGHT("@(#) Copyright (c) 1991, 1993\n\
 #if 0
 static char sccsid[] = "@(#)main.c	8.7 (Berkeley) 7/19/95";
 #else
-__RCSID("$NetBSD: main.c,v 1.33 1999/03/27 13:46:19 christos Exp $");
+__RCSID("$NetBSD: main.c,v 1.40 2001/02/04 19:52:06 christos Exp $");
 #endif
 #endif /* not lint */
 
+#include <errno.h>
 #include <stdio.h>
 #include <signal.h>
 #include <sys/stat.h>
@@ -84,16 +85,16 @@ int rootpid;
 int rootshell;
 STATIC union node *curcmd;
 STATIC union node *prevcmd;
-extern int errno;
 #if PROFILE
 short profile_buf[16384];
 extern int etext();
 #endif
 
-STATIC void read_profile __P((char *));
+STATIC void read_profile __P((const char *));
 STATIC char *find_dot_file __P((char *));
 int main __P((int, char **));
 
+extern int oexitstatus;
 /*
  * Main routine.  We initialize things, parse the arguments, execute
  * profiles if we're a login shell, and then call cmdloop to execute
@@ -209,7 +210,7 @@ state3:
 	}
 
 	if (minusc)
-		evalstring(minusc);
+		evalstring(minusc, 0);
 
 	if (sflag || minusc == NULL) {
 state4:	/* XXX ??? - why isn't this before the "if" statement */
@@ -266,12 +267,13 @@ cmdloop(top)
 			evaltree(n, 0);
 		}
 		popstackmark(&smark);
+		setstackmark(&smark);
 		if (evalskip == SKIPFILE) {
 			evalskip = 0;
 			break;
 		}
 	}
-	popstackmark(&smark);		/* unnecessary */
+	popstackmark(&smark);
 }
 
 
@@ -282,7 +284,7 @@ cmdloop(top)
 
 STATIC void
 read_profile(name)
-	char *name;
+	const char *name;
 {
 	int fd;
 	int xflag_set = 0;
@@ -346,7 +348,7 @@ find_dot_file(basename)
 	char *basename;
 {
 	char *fullname;
-	char *path = pathval();
+	const char *path = pathval();
 	struct stat statb;
 
 	/* don't try this for absolute or relative paths */
@@ -401,8 +403,6 @@ exitcmd(argc, argv)
 	int argc;
 	char **argv;
 {
-	extern int oexitstatus;
-
 	if (stoppedjobs())
 		return 0;
 	if (argc > 1)
